@@ -5,24 +5,15 @@ use std::collections::HashMap;
 use std::path::{Path, PathBuf};
 
 use flatbuffers::{ForwardsUOffset, Vector, WIPOffset};
-use geo::Coord;
+use geo::{Coord, Distance};
 
 use crate::curves::{Curve, CurveError, CurveProjection, SphericalLineStringCurve};
 
+use crate::lrs::Properties;
+use crate::lrs_ext::ExtLrs;
 use crate::lrs_generated::{self, *};
 use crate::osm_helpers::sort_edges;
-
-/// A key-value `HashMap` to add metadata to the objects.
-pub type Properties = std::collections::HashMap<String, String>;
-
-#[macro_export]
-/// Build a properties map:
-/// `properties!("source" => "openstreetmap", "licence" => "ODbL")`.
-macro_rules! properties {
-    ($($k:expr => $v:expr),* $(,)?) => {{
-        core::convert::From::from([$(($k.to_owned(), $v.to_owned()),)*])
-    }};
-}
+use crate::properties;
 
 /// The linear position of an [`Anchor`] doesn’t always match the measured distance.
 /// For example if a road was transformed into a bypass, resulting in a longer road,
@@ -405,6 +396,11 @@ impl<'fbb> Builder<'fbb> {
         self.fbb.finished_data()
     }
 
+    /// Builds the LRS from the data.
+    pub fn build_lrs(&mut self, properties: Properties) -> Result<ExtLrs, String> {
+        ExtLrs::load(self.build_data(properties))
+    }
+
     /// Return the mapping between a traversal id and its index in the builder.
     pub fn get_traversal_indexes(&self) -> HashMap<String, usize> {
         self.temp_traversal
@@ -477,13 +473,13 @@ impl<'fbb> Builder<'fbb> {
         }
     }
 
-    /// Gives the euclidian distance between two traversals
+    /// Gives the euclidean distance between two traversals
     /// While working on spherical coordinates, this usually doesn’t make much sense,
     /// this is good enough to sort curves by distance
     pub fn euclidean_distance(&self, lrm_index_a: usize, lrm_index_b: usize) -> f64 {
         let a = &self.temp_traversal[lrm_index_a].curve.geom;
         let b = &self.temp_traversal[lrm_index_b].curve.geom;
-        geo::EuclideanDistance::euclidean_distance(a, b)
+        geo::Euclidean::distance(a, b)
     }
 
     /// Returns the position along the curve of the traversal
@@ -509,7 +505,7 @@ impl<'fbb> Builder<'fbb> {
 
     /// Orient the traversal according to two points
     ///
-    /// In the end, the first coordinate must be closer to the begining than the second
+    /// In the end, the first coordinate must be closer to the beginning than the second
     /// If both points are so far from the curve that they are projected to a end, we consider the offset to the curve
     pub fn orient_along_points(
         &mut self,
@@ -531,7 +527,10 @@ impl<'fbb> Builder<'fbb> {
 
 #[cfg(test)]
 mod tests {
+    use crate::lrm_scale::LrmScaleMeasure;
+
     use super::*;
+    use approx::assert_relative_eq;
     use geo::{coord, point};
 
     fn build_traversal(builder: &mut Builder) -> usize {
@@ -614,4 +613,53 @@ mod tests {
         b.orient_along_points(traversal, point_b, point_a).unwrap();
         assert_eq!(b.temp_traversal[traversal].segments[0].segment_index, 1);
     }
+
+    #[test]
+    fn convert_to_lrs() {
+        let mut b = Builder::new();
+        let traversal = build_traversal(&mut b);
+        let anchor_index =
+            b.add_anchor("bridge", Some("bridge"), coord! {x:0., y:0.}, properties!());
+        let anchor_2_index =
+            b.add_anchor("foo", Some("bridge"), coord! {x:0.5, y:0.}, properties!());
+        let aol = AnchorOnLrm {
+            anchor_index,
+            distance_along_lrm: 1000.,
+        };
+
+        let aol2 = AnchorOnLrm {
+            anchor_index: anchor_2_index,
+            distance_along_lrm: 2000.,
+        };
+        b.add_lrm("lrm", traversal, &[aol, aol2], properties!());
+
+        let lrs = b.build_lrs(properties!()).unwrap();
+        let lrm = lrs
+            .resolve(
+                0,
+                &LrmScaleMeasure {
+                    anchor_name: "bridge".to_owned(),
+                    scale_offset: 0.,
+                },
+            )
+            .unwrap();
+        assert_relative_eq!(lrm.x(), 0.);
+        assert_relative_eq!(lrm.y(), 0.);
+    }
+
+    #[test]
+    fn properties() {
+        let mut b = Builder::new();
+        let traversal = build_traversal(&mut b);
+        b.add_lrm(
+            "lrm",
+            traversal,
+            &[],
+            properties!("test_key" => "test_value"),
+        );
+        let lrs = b.build_lrs(properties!()).unwrap();
+
+        let p = lrs.lrm_properties(0);
+        assert_eq!(p["test_key"], "test_value")
+    }
 }
